#include <math.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_view.h>
#include <uf_draw.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_drawing_views 
*
*  Place the FRONT, RIGHT, TOP and TFR-ISO views on the drawing.
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                  
*   01   01JUL96   J_P   Added calls to set view drawing ref pt.
*   02   14FEB03   DKF   Minor corrections,
*                        Updated error checking,
*                        removed unreferenced local variable
*
* input 
*    dname                 drawing name 
*    dsize                 drawing size 
*
* output 
* calc_drawing_views        0=success, else failure 
*  
*  
**********************************************************************/

    int calc_drawing_views(char *dname,int dsize)
{
/******************** Routine variable declaration  *******************/

    tag_t
        view_tag;       /* Tag of TOP@ view for border resizing */

    double
        refpt[2],       /* drawing reference point */
        height,         /* calculator height       */
        width,          /* calculator width        */
        hspace,         /* horizontal space between views */
        vspace,         /* vertical space between views */
        center[3],      /* new view center */
		fraction=1.0;   /* fit per cent for UF_VIEW_fit_view */

    double
        htwd[][2] =  { 841.0, 1189.0,
                       594.0,  841.0,
                       420.0,  594.0,
                       297.0,  420.0,
                       210.0,  297.0 };

/******************** Beginning of Executable code *********************/


/* Evaluate the expressions for calculator XXX height and XXX width.
 * Check return codes.
 */

/* Calculate spacing between views */
    hspace = (htwd[dsize-1][1] - width - height)/3.0;
    vspace = (htwd[dsize-1][0] - height - 12.0)/3.0;

/*XXX Set the View center and XXX view's drawing ref. for the FRONT
 * view.
 */
    center[0] = 0.5*width;
    center[1] = 0.5*height;
    center[2] = 6.0;
    
/*XXX Import (add) the FRONT view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = hspace + width/2.0;
    refpt[1] = vspace + 6.0;

/*XXX Set the View center and XXX view's drawing ref. for the RIGHT view. */


/*XXX Import (add) the RIGHT view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = htwd[dsize-1][1] - hspace - 0.5*height;
    refpt[1] = vspace + 6.0;


/*XXX Set the View center and XXX view's drawing ref. for the TOP view. */


/*XXX Import (add) the TOP view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = hspace + width/2.0;
    refpt[1] = htwd[dsize-1][0] - vspace - 0.5*height;


/* Change the clipping bounds on the TOP view.  */
    flag = uc6485("TOP@",borders);
    borders[0] = 0.5 * borders[0];
    borders[1] = 0.5 * borders[1];
    borders[2] = 2.0 * borders[2];
    borders[3] = 2.0 * borders[3];
    flag = uc6486("TOP@3",borders);

/*XXX Set the View center and XXX view's drawing ref. for the TFR-ISO view. */


/*XXX Import (add) TFR-ISO view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = htwd[dsize-1][1] - hspace - 0.5*height;
    refpt[1] = htwd[dsize-1][0] - vspace - 0.5*height;


/*XXX Fit the Work View at 100% */

    return(0);
}
