#include <string.h>
#include <math.h>
#include <uf_defs.h>
#include <uf_obj.h>
#include <uf_modl.h>
#include <uf_attr.h>
#include <uf_part.h>
#include "calcproto.h"

/**********************************************************************
* calc_model_bottom 
* 
* create the bottom half of the calculator 
*  
* revisions 
* rv   date      who   reason for change 
* 00   01NOV95   J_P 
* 01   01JUN98   J_P   Initialized some variables
* 02   14FEB03   DKF   Removed unreferenced local variable
* 03   14FEB03   DKF   Updated part attribute assignment procedure
*  
* input 
*  (none) 
*  
* output 
*  calc_model_bottom        0 = success, >0 = error 
*  
**********************************************************************/


    int calc_model_bottom(void)
{
/******************** Routine variable declaration  *******************/
    char
        *edge_len_p[3],     /* Pointers to block edge length strings. */
        edge_len[3][133],   /* Strings for edge length expressions. */
        svalue[50];         /* string for attribute value */

    int ecount=0,           /* count of objects in list */
        vcount=0,           /* count of vertices in edge */
        flag=0,             /* 0 = success, >0 = error */
        i;                  /* looper */

    int allow_smooth=0,  /* allow all blends */
        allow_cliff=0,
        allow_notch=0;

    tag_t
        bottom_half,        /* bottom half feature */
        body,               /* bottom_half body    */
        hollow,             /* hollow feature */
        blend1,             /* blend feature */
        part;               /* part tag, for attribute assignment */

    uf_list_p_t
        list1=NULL,         /* list of edges */
        list2=NULL;

    double
        corner[3],          /* start corner of block */
        pt1[3],             /* coordinates of vertex */
        pt2[3],             /* coordinates of vertex */
        vrb_tol=0.001;      /* variable radius blend tolerance */


    UF_ATTR_value_t
        att_value;          /* attribute data structure */

/******************** Beginning of Executable code *********************/


/* Set up data for block creation.  The routine takes an array of
 * pointers to strings.  Set edge_len_p to the addresses of the
 * edge_len array items.
 */

    corner[0] = 0.0;
    corner[1] = 0.0;
    corner[2] = 0.0;

    edge_len_p[0] = &edge_len[0][0];
    edge_len_p[1] = &edge_len[1][0];
    edge_len_p[2] = &edge_len[2][0];

    strcpy(edge_len[0],"Body_width");
    strcpy(edge_len[1],"Body_height");
    strcpy(edge_len[2],"5.0");

/*XXX Create the block and check the return code */


/*XXX Obtain the body tag from the feature tag. */


/*XXX Obtain the body edges. Use list1 */


/*XXX Determine the number of edges (ecount). Trap the return code */
    

/*XXX Delete the list and return if an error occurs */


/*XXX Create list2' for storing the edges to blend. Trap the return code */


/*XXX If an error occurs, delete list1 and return. */


/* Loop through all edges and get their vertices.  Use the edge
 * vertices to determine which edges to blend.  The edges we want have a
 * delta in Z of 5mm.  Put the id's of the edges to blend in 'list2'. 
 */
    for(i=0; i < ecount; i++) {
        tag_t
            edge;               /* edge object */

		/*XXX Get an edge from the list.  Check return code.  */

		/*XXX Obtain the vertices.  Check return code. */

        if(fabs(fabs(pt1[2] - pt2[2]) - 5.0) < 0.001) {
			/*XXX Item is placed on list2.  Check return code. */
        }
    }

/*XXX Blend the qualified edges with a 5 mm radius.  Check
 * return code.
 */

/*XXX Delete the previous edge lists before blending the edges of the
 * bottom face.  Since the block has been edited (blended), the first
 * edge list may no longer be valid.
 */

/*XXX Get the new body edges. Use list1*/

/*XXX Obtain the number of objects on the edge list (ecount). 
 * Trap the return code */

/*XXX Delete the list and return if an error occurs */

/*XXX Create 'list2' for storing edges to blend. Trap return code.
 */

/*XXX If an error occurs, delete list1 and return. */


    for(i=0; i < ecount; i++) { 
        tag_t
            edge;               /* edge object */

		/*XXX Obtain an edge from the list.  Check return code. */

		/*XXX Get the edge vertices and check them.  We want bottom edges
		* with Z of 0.
		*/

        if((fabs(pt1[2]) < 0.001)
            && (fabs(pt2[2]) < 0.001)) { 
			/*XXX Item is placed on list2.  Check return code. */
	
        }
    }

/*XXX Blend the qualified edges with a 2 mm radius.  Check
 * return code.
 */

/*XXX Delete the lists used for the blend. */

/*XXX Get the body faces. Use list1 */

/*XXX Create list2 for storing the face to hollow. */


/*XXX Get the count of faces (ecount).  Loop through the faces and
 * check the Z coordinate of the center and the normal.
 */

    for(i=0; i < ecount; i++) {
        tag_t
            face;               /* face object      */

        int ftype,              /* face type */
            dsense;             /* face direction sense */

        double
            nrm[3],             /* Face normal direction */
            box[6],             /* face boundary */
            rad1,               /* face major radius */
            rad2;               /* face minor radius */

		/*XXX Obtain an face from the list.  Check return code. */

		/*XXX Obtain data about the face.  Check return code. */

        if((fabs(pt1[2] - 5.0) < 0.001)
            && (fabs(nrm[2] - 1.0) < 0.001)) {
			/*XXX Put face on list2.  Check return code.  Break from the loop. */

            break;
        }
    }

/*XXX Perform hollow operation.  Use a 1mm thickness.
 * Check return code.
 */

/*XXX Delete the lists used to create the hollow. */

/*XXX Assign the name BOTTOM_HALF to the body.  */

/*YYY  Attribute code will be added after the model code is completed */
/* Add part number and description attributes to the part.  Obtain the 
 * display part tag. .
 * For UF_ATTR_assign, use the titles
 * "PARTNUMBER" and "DESCRIPTN"
 * and the values shown below.
 */
/*YYY Obtain the part tag for the current part */

    att_value.type = UF_ATTR_string;
    strcpy(svalue,"002");    /* First call for the part number */
    att_value.value.string = svalue;
/*YYY Assign the attribute */

    strcpy(svalue,"BOTTOM");    /* Second call for the description. */
/*YYY Assign the attribute */


    return (0);
}
