#include <string.h>
#include <math.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_object_types.h>
#include "calcproto.h"

/**********************************************************************
* calc_model_top_holes
*
*   This routine cuts the pockets in the calculator top for the lcd, 
* photocell, and buttons.
*
* revisions 
* rv   date      who   reason for change 
* 00   01NOV95   J_P 
* 01   01AUG97   J_P   changed some variable names and limited scope of
*                      other variables.
* 02   01JUN98   J_P   initialized some variables
* 03   14FEB03   DKF   Removed unreferenced local variables
*
* input 
*  body                   Body of top half of calculator 
*  
* output
*  holes                  List of photocell, lcd, and button pockets
*  calc_model_top_holes   0 = success,
*                         1 = Error occurred.   Message handled here.
*                         >1 = error
*  
*  
**********************************************************************/

    int calc_model_top_holes(tag_t body, uf_list_p_t holes)
{
/******************** Routine variable declaration  *******************/
    char
        *poclen_p[3],   /* Pointers to pocket length exp. strings */
        poclen[3][133], /* Strings for pocket length expressions  */
        bspacex[132],   /* String for inst. array XC spacing exp. */
        bspacey[132];   /* String for inst. array YC spacing exp. */

    int count=0,        /* count of objects in list */
        i=0;

    uf_list_p_t
        list1=NULL;     /* list of edges or features to instance */

    tag_t
        face1,          /* top face for hole placement */
        feat1;          /* pocket or hole feature */

    double
        location[3],    /* location for hole or pocket feature */
        axis[3],        /* tool axis (pocket 'direction') */
        x_dir[3],       /* feature direction     */
        body_height=0,  /* Numeric values of expression */
        body_width=0,   /* Numeric values of expression */
        button_height=0,/* Numeric values of expression */
        button_width=0, /* Numeric values of expression */
        lcd_pc_height=0,/* Numeric values of expression */
        lcd_height=0,   /* Numeric values of expression */
        pc_height=0;    /* Numeric values of expression */

/******************** Beginning of Executable code *********************/

/*XXX Get the body faces. XXX Obtain the count of faces (count) */

/* Loop through the faces and check the face_data for a planar face
 * (UF_bounded_plane_subtype) with a normal in the +Z
 */
    for(i=0; i < count; i++) { 
        tag_t
            face;

        int ftype,          /* face type */
            dsense;         /* face direction sense */

        double
            pt1[3],         /* coordinates of vertex */
            nrm[3],         /* Face normal direction */
            box[6],         /* face boundary         */
            rad1,           /* face major radius     */
            rad2;           /* face minor radius     */

/*XXX Extract a face from the list.  Check the return code.  */

/*XXX Obtain the face data.  Check return code.  */

        if(ftype == UF_bounded_plane_subtype) {
            if(fabs(nrm[2] - 1.0) < 0.001) {
                face1 = face;
                break;
            }
        }
    }

/*XXX Delete the face list and check the return code.  */

/*XXX Evaluate the expressions for Body_width, Body_height, pc_height,
 * lcd_height, lcd_pc_height, Button_width, and Button_height.  Check
 * return codes.
 */

/* Set the location for the photocell pocket, it's depth direction (-Z),
 * and it's orientation direction (X).
 */

    location[0] = 0.5*body_width;
    location[1] = body_height - 5.0 - 0.5*pc_height;
    location[2] = 10.0;

    axis[0] = 0.0;
    axis[1] = 0.0;
    axis[2] = -1.0;

    x_dir[0] = 1.0;
    x_dir[1] = 0.0;
    x_dir[2] = 0.0;

/* Set the character pointers to the expression string variables.
 * Copy in the expression and use "2.0" for depth.
 */
    poclen_p[0] = &poclen[0][0];
    poclen_p[1] = &poclen[1][0];
    poclen_p[2] = &poclen[2][0];

    strcpy(poclen[0],"pc_width");
    strcpy(poclen[1],"pc_height");
    strcpy(poclen[2],"2.0");

/*XXX Create the photocell pocket.  Use a corner and floor radius of "0.0"
 * and a taper angle of 1.5.  Check the return code. 
 */

/*XXX Put the pc pocket feat on the list passed as 'holes' in the function
 * calling argument.
 */

/* Change the location for the liquid crystal display.  */
    location[1] = location[1] - 0.5*pc_height - lcd_pc_height
                  - 0.5*lcd_height;

/* The expressions are the same except for height.  */
    strcpy(poclen[1],"lcd_height");

/*XXX Create the lcd pocket.  Use a corner and floor radius of "0.0" and
 * a taper angle of 1.5.  Check the return code. 
 */

/*XXX Put the lcd pocket feat on the list passed as 'holes' in the function
 * calling argument.
 */

/* Set the size for the buttonhole.  The depth is the same as the other
 * pockets.  The width has an extra .5 mm for clearance. 
 */
    strcpy(poclen[0],"Button_width+0.5");
    strcpy(poclen[1],"Button_height+0.5");

/* Locate the first button in the lower left corner.  All others will 
 * be placed from that one.
 */
    location[0] = 5.0 + 0.5*button_width;
    location[1] = 5.0 + 0.5*button_height;

/*XXX Create the 1st button pocket.  Use a corner and floor radius of
 * "0.0" and a taper angle of 1.5.  Check the return code. 
 */

/* Set up the spacing expressions for the linear instance array. */
    strcpy(bspacex,"Button_spacing+Button_width");
    strcpy(bspacey,"Button_spacing+Button_height");

/*XXX Put the button hole feat on the list passed as 'holes' in the
 * function calling argument.
 */

/*XXX Create a local list and XXX put the 1st button hole on the list.
 * Check the return code.
 */

/*XXX Create the linear instance set.  Use the expression "Across" for
 * the number of features in the XC direction and "Up" for the number
 * of features in the YC direction.  Use the bspacex/bspacey strings
 * for the spacing.  Check the return code. 
 */

/*XXX Delete the list created for the iset call.  Check return code.  */

    return(0);
}
