/************************************************************************
*   calc_set_style_dialog
*
* This routine updates the calculator style dialog when the user selects
* a style.  The first time it is selected, it activates the OK button.
* On subsequent calls, it stores the current height/width and replaces
* those values with the height/width for the new style.  Static variables
* are used to retain values between calls to this function.
*
*  input
*    dialog_id    int  Identifier of the dialog to alter.
*    style        int  Style of the calculator 
*                      1 = Vertical
*                      2 = Horizontal
*
*  output
*    (none)
*  revisions
*    rv    date        who    reason for change
*    00    01NOV95     J_P
*    01    14FEB03     DKF    Update error checking
*
************************************************************************/

#include <stdio.h>
#include <uf_defs.h>
#include <uf_styler.h>
#include "calc_setup.h"

extern void calc_set_style_dialog( int dialog_id, int style )
{
/********************* Routine variable declaration  ********************/
    static double
        vert_height=140.;       /* Height for Vertical style calculator.*/
    static double
        vert_width=72.5;        /* Width for Vertical style calculator.*/

    static double
        horz_height=100.;       /* Height for Horiz. style calculator.*/
    static double
        horz_width=150.;        /* Width for Horiz. style calculator.*/

    UF_STYLER_item_value_type_t
        okb,                    /* Structure to change the OK button 
                                   sensitivity. */
        dlg;                    /* Dialog structure to ask/set values */
    
    static int
        has_ok = FALSE;         /* Flag that OK is sensitive. */

/******************** Beginning of Executable code **********************/


/* If the OK button flag has_ok is FALSE, this is the first style
 *  pick.  Set the real values (to the default of 140 by 72.5 or
 * 100 by 150) and make the them sensitive.
 */
    if( !has_ok ) {
        has_ok = TRUE;
        /* Set the height based on the style selected.
         * Make the item sensitive too.
         */
        dlg.item_attr = UF_STYLER_VALUE;
        dlg.item_id = CALC_HEIGHT;
        if( 1 == style )
            dlg.value.real = vert_height;
        else 
            dlg.value.real = horz_height;
        UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );

        dlg.item_attr = UF_STYLER_SENSITIVITY;
        dlg.value.integer = TRUE;
        UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );


        /* Set the width based on the style selected. 
         * Make the item sensitive too.
         */
        dlg.item_attr = UF_STYLER_VALUE;
        dlg.item_id = CALC_WIDTH;
        if( 1 == style )
            dlg.value.real = vert_width;
        else 
            dlg.value.real = horz_width;
        UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );

        dlg.item_attr = UF_STYLER_SENSITIVITY;
        dlg.value.integer = TRUE;
        UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );

        /* Set the OK button to be sensitive. */
        okb.item_attr = UF_STYLER_SENSITIVITY;
        okb.item_id = UF_STYLER_NAV_INDEX;
        okb.subitem_index = UF_STYLER_OK_INDEX;
        okb.value.integer = TRUE;
        UF_CALL( UF_STYLER_set_value( dialog_id, &okb ) );

    }
    else {
    /* If the new style is vertical, the current number should
     * replace the old horizontal value.  Once the old value 
     * is obtained, the real value is swapped for the other
     * style's value.
     */
        dlg.item_attr = UF_STYLER_VALUE;
        dlg.item_id = CALC_HEIGHT;
        UF_CALL( UF_STYLER_ask_value( dialog_id, &dlg ) );
        if( 1 == style ) {
            horz_height = dlg.value.real;
            dlg.value.real = vert_height;
            UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );
        }
        else {
            vert_height = dlg.value.real;
            dlg.value.real = horz_height;
            UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );
        }

        dlg.item_attr = UF_STYLER_VALUE;
        dlg.item_id = CALC_WIDTH;
        UF_CALL( UF_STYLER_ask_value( dialog_id, &dlg ) );
        if( 1 == style ) {
            horz_width = dlg.value.real;
            dlg.value.real = vert_width;
            UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );
        }
        else {
            vert_width = dlg.value.real;
            dlg.value.real = horz_width;
            UF_CALL( UF_STYLER_set_value( dialog_id, &dlg ) );
        }
    }
}
