#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <uf.h>
#include <uf_cfi.h>
#include <uf_part.h>
#include <uf_assem.h>
#include <uf_attr.h>
#include <uf_draw.h>

#define    UF_CALL(X) (report( #X, __FILE__, __LINE__, (X) ))

static int report( char *call, char *file, int line, int irc )
{
    if( irc ) {
        char messg[133];
        printf("%s\n%s, line %d:  ", call, file, line );
        (UF_get_fail_message( irc, messg )) ? 
            printf("Returned %d\n", irc ):
            printf("Error %d:  %s\n", irc, messg );
    }
    return( irc );
}

/**********************************************************************
*
* calcx
*
* This external user function obtains part numbers and descriptions 
* from the calculator assembly.  The part file name and the output 
* file (for part numbers/description data) are required.
*
* revisions
*   rv   date      who reason for change
*   00   01NOV95   J_P
*   01   14FEB03   DKF  removed obsolete #include <unistd.h> and 
*                       added substitute code for Windows 
*
* input
*  argc                number of argument
*  argv                array of pointers to the arguments
*
* output
*                      0 = success, >0 = error
*  
*  
**********************************************************************/

    main(int argc,char *argv[])
{
/******************** Routine variable declaration  *******************/
    char
        msg[132],                     /* message */
        svalue[132],                  /* attribute string value */
        fspec[MAX_FSPEC_SIZE+1],      /* name of child part */
        dspec[MAX_FSPEC_SIZE+1],      /* directory part of fspec */
        cname[30],                    /* part name of child */
        pname[30],                    /* part name without ".prt" */
        savename[30];                 /* part of previous pass */

    int
        flag,                         /* general purpose flag */
        fid;                          /* file channel number */

      tag_t
        top_part,                     /* tag of the top part */
        inst,                         /* tag of instance */
        child,                        /* tag of child part */
        atteid;                       /* part attribute entity */

    UF_PART_load_status_t
        estat;                        /* error status structure */

    UF_ATTR_value_t
        att_value;                    /* attribute data structure */

/******************** Beginning of Executable code *********************/


/* check input */
    if(argc != 3) {
        strcpy(msg,"This program takes two arguments: ");
        strcat(msg,"the root part file name and the name\n" );
        strcat(msg,"of the output text file.");
        printf("%s\n",msg);
        exit(1);
     }

/*XXX Check out a user function license */

/*XXX Open the part.  It should be argv[1].  Store it's tag as top_part. */
    strcpy(fspec, argv[1] );

/*XXX Call the routine to set the view to a model view (if a drawing
 * is displayed).  It has no effect if a model view is active.
 * This must be done to allow changes to the work part.  (It cannot
 * be changed when in a drawing) .
 */

/* Open the text file (in argv[2]) using fid for file identifier.  */
    fid = uc4504(argv[2],3,0);
    if(fid < 0) {
        UF_get_fail_message(flag,msg);
        printf("%s\n",msg);
        exit(4);
    }
/* Write a header "Report for calculator" to the file id. */
    uc4524(fid,"Report for Calculator");
    uc4524(fid,"---------------------");

/* Set the attribute char pointer to point to the svalue character string
 * array.  Initialize the savename to a NULL string.
 */
    att_value.value.string = svalue;
    strcpy(savename,"");

/* Loop through all the component instances.  For each unique instance,
 * obtain the attribute information and write it to the file.
 */
    inst = NULL_TAG;
    while(TRUE) {


/*XXX Set inst to the next instance under the top_part using the assem 
 * routine to cycle instances.  Get the part names of the instances.
 */
        if( NULL_TAG == inst )
            break;
        UF_ASSEM_ask_part_name_of_child(inst,fspec);

/* Split the file spec into the directory and part name strings. Remove
 * the '.prt' from the part file string and save the result in pname.
 */
        uc4576(fspec,2,dspec,cname);
        uc4578(cname,2,pname);

/* Check if pname is the same as the previous name.  Skip it if it is (the
 * buttons will have the same name and should only be done once.
 */
        if(strcmp(savename,pname) == 0) {
            continue;
        }
        printf("Working on %s\n",pname);

/* Write a blank line and then the file name.  */
        uc4524(fid," ");
        uc4524(fid,pname);

/*XXX Set child to the child part tag of the instance and XXX change
 * work part to the child so we can get the part attribute.
 * Check return code.
 */

/*XXX Get the part attribute ID and read the PARTNUMBER and DESCRIPTN
 * attribute values.
 */
        sprintf(msg,"   Part Number = %s",svalue);
        uc4524(fid,msg);

        sprintf(msg,"   Description = %s",svalue);
        uc4524(fid,msg);

/* Copy the name of the part we just did to the save string */
        strcpy(savename,pname);
    }

/* Close and print the file */
    uc4540(fid,0);
/* Unix */
    sprintf(msg,"cat %s",argv[2]);
/* Windows 
    sprintf(msg,"type %s",argv[2]);*/
    system(msg);

/*XXX Release user function license */
    UF_CALL( UF_terminate() );
    exit(0);
}
