/*===========================================================================
 
                  Copyright (c) 1997 Electronic Data Systems Corporation
                             Unpublished - All rights reserved
 
 ===========================================================================*/
#include <Xm/RowColumn.h>
#include <Xm/MainW.h>
#include <Xm/Command.h>
#include <Xm/FileSB.h>
#include <Xm/Text.h>
#include <string.h>
#include <uf_defs.h>
#include <uf.h>
#include <uf_ui.h>
#include <uf_ui_xt.h>

/******************************************************************************/
/*       Define variables used in this set of routines.                       */

#define FSB_TITLE    "Select a File"
/******************************************************************************/
  
/******************************************************************************/
/*    Protoype internal routines.  Make then static, if possible.             */
   
static void GetFileName( Widget, int, XmFileSelectionBoxCallbackStruct *);
static void GetFileCancelled( Widget, int, XmFileSelectionBoxCallbackStruct *);
extern void get_filename( char *, char * );
    
/******************************************************************************/


/******************************************************************************/
/*      Define Global Variable and declare externals                          */
char    *filename_p;    /* Global character pointer.  It will point to the 
                           output string array.  It can be accessed in any
                           routine in this source file.*/
/******************************************************************************/


/******************************************************************************
*  This routine takes a character pointer as input and outputs data to another
*  character pointer.  The return character string is not greater than
*  MAX_FSPEC_SIZE+1.  The routine displays the Motif file selection dialog
*  box using the first input parameter as the filter.  If nothing is selected,
*  the routine returns the string "CANCEL" (upper case).
******************************************************************************/
extern void get_filename( char *filter_p,
                          char *outfilename_p )
{
    Widget
        parent_widget,
        FileBox;

    Boolean
        flag=FALSE;

    XmString
        filter_string;


/* Initialize string variables.  */
    strcpy( outfilename_p, "CANCEL");
    filename_p = outfilename_p;

/* Create Motif dialog under UG.  Get the parent widget (UG). */
    parent_widget = UF_UI_get_default_parent();

/* Create file selection dialog widget.  Set the filter string and turn off
 * the default positioning.  Set the position of the parent (shell) to be on
 * the left of the screen.
 */
    FileBox = XmCreateFileSelectionDialog(parent_widget,"FileBox", NULL, 0);
    filter_string = XmStringCreateSimple( filter_p );
    XtVaSetValues(FileBox, XmNpattern, filter_string, XmNdefaultPosition, flag,
                  NULL );
    XtVaSetValues(XtParent(FileBox), XmNx, 0, XmNy, 80, XtVaTypedArg,
                  XmNtitle, XmRString, FSB_TITLE, strlen(FSB_TITLE)+1, NULL );


/* Add callback functions */
    XtAddCallback( FileBox, XmNcancelCallback,
                (XtCallbackProc) GetFileCancelled, NULL );
    XtAddCallback( FileBox, XmNokCallback, (XtCallbackProc)GetFileName,
                NULL );

/* Get rid of HELP */
    XtUnmanageChild(XmFileSelectionBoxGetChild(FileBox,XmDIALOG_HELP_BUTTON));

/* Manage the widget */
    XtManageChild(FileBox);

/* Run the widget */
    UF_UI_run_dialog (FileBox);
}

/******************************************************************************
*  This routine is the callback when OK is selected.  The routine copies the
*  data from the key-in (value) area of the widget from the temporary Motif
*  memory area to the output file pointer.  The 'Motif' memory is freed.
******************************************************************************/
static void    GetFileName( 
    Widget    widget,
    int        client_data,
    XmFileSelectionBoxCallbackStruct    *CallbackStruct )
{
    char
        *temp_filename;

/* Extract the filename from the value area */
    XmStringGetLtoR( CallbackStruct->value, XmSTRING_DEFAULT_CHARSET,
            &temp_filename );

/* Unmanage the file selection box */
    XtUnmanageChild( widget );

/* pass return string.  NOTE:  No error checking done.  Hitting OK
 * without picking a file will return the selection path, not a NULL
 * or specific string.
 */
    strcpy( filename_p, temp_filename );

/* Free the pointer.  Space was allocated by GetLtoR. */
    XtFree(temp_filename);

/* Kill the dialog */
    UF_UI_exit_dialog ();
}

/******************************************************************************
*  This routine is the callback when CANCEL is selected.  The routine sets
*  the global string to be "CANCEL" and unmanages and exists the dialog. 
******************************************************************************/
static void GetFileCancelled( Widget widget,
                              int client_data,
                              XmFileSelectionBoxCallbackStruct *CallbackStruct )
{

/* Unmanage the file selection box */
    XtUnmanageChild( widget );

/* Pass Cancel back */
    strcpy( filename_p, "CANCEL" );

/* Call UG function to exit */
    UF_UI_exit_dialog ();
    
}
