#include <string.h>
#include <uf_defs.h>
#include <uf_part.h>
#include <uf_assem.h>
#include <uf_layer.h>
#include "calcproto.h"

/**********************************************************************
*   Set up the components for the different parts in the calculator
* of the calculator 
*
*  Revisions 
*   rv    date       who    reason for change 
*   00    01Nov95    J_P                   
*   01    14Feb03    DKF    update error checking, add Windows paths                   
*
* input 
*   (none) 
*
* output 
*   top_inst     Tag of top half instance of calculator 
*   bottom_inst  Tag of bottom half instance 
*   button_inst  Tag of button instance 
*   calc_assem   0 = success, > 0 = error   
*  
*  
**********************************************************************/
      int calc_assem(tag_p_t top_inst_p, tag_p_t bottom_inst_p,
                     tag_p_t button_inst_p)
{
/******************** Routine variable declaration  *******************/
    char
        pname[MAX_FSPEC_SIZE+1],        /* new part name */
        refset[MAX_ENTITY_NAME_SIZE+1], /* reference set name */
        iname[MAX_ENTITY_NAME_SIZE+1];  /* name of comp inst */

    tag_t
        workp,      /* current work part */
        nullt=NULL_TAG;

    int units,      /* 1 = millimeters, 2 = inches */
        i;          /* loop variable */

    const int
        zero=0, one=1, two=2;

    double
        imat[]={1.0, 0.0, 0.0,
                0.0, 1.0, 0.0 },    /* orientation for components */
        origin[]={0.0, 0.0, 0.0};   /* origin for each component */


/******************** Beginning of Executable code *********************/

/* Set the workp variable to the tag of the current work part.  Return
 * an error if there is no work part. 
 */
    workp = UF_ASSEM_ask_work_part();
    if( NULL_TAG == workp )
        return( 1 );

/* Obtain the units of the work part.  Return an error if the units
 * are english.
 */
    UF_CALL( UF_PART_ask_units( workp, &units ) );
    if( UF_PART_ENGLISH == units )
        return( 2 );

/* Turn on layers 1 through 3 (Make them 'active') */
    for( i=1; i<4; i++ ) {
        UF_CALL(UF_LAYER_set_status( i, two ));
    }


/* Create the component part called 'top_half'.  Use the entire
 * part (set the reference set name to a null string).  The component
 * part creation function takes a list of objects.  Pass a nullt (a
 * NULL_TAG) to create a part with no geometry.
 */
	/* Windows */
    strcpy(pname,".\\top_half");
	/* Unix 
    strcpy(pname,"./top_half");*/
    refset[0] = '\0';
    strcpy(iname,"TOP_HALF");
    UF_CALL( UF_ASSEM_create_component_part( workp, pname, refset,
             iname, units, one, origin, imat, zero, &nullt, top_inst_p) );

/* Create the component part called 'bottom_half'.  Use the entire
 * part (the reference name is still set to a null string).
 */
	/* Windows */
    strcpy(pname,".\\bottom_half");
	/* Unix 
    strcpy(pname,"./bottom_half");*/
    strcpy(iname,"BOTTOM_HALF");
    UF_CALL( UF_ASSEM_create_component_part( workp, pname, refset,
             iname, units, one, origin, imat, zero, &nullt, bottom_inst_p) );

/* Create the component part called 'buttons'.  Use the entire
 * part (the reference name is still set to a null string).
 */
	/* Windows */
    strcpy(pname,".\\button");
	/* Unix 
    strcpy(pname,"./button");*/
    strcpy(iname,"BUTTON");
    UF_CALL( UF_ASSEM_create_component_part( workp, pname, refset,
             iname, units, one, origin, imat, zero, &nullt, button_inst_p) );

    return (0);
}
