#include <stdio.h>
#include <string.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include "calcproto.h"

/**********************************************************************
* calc_expr will create the fundamental set of expressions that
* control construction of the calculator.
*
*  revisions 
*  rv     date      who   reason for change 
*  00   01NOV95     J_P                   
*  01   14FEB03     DKF   Updated error  checking
*
*  input 
*    style        1 = vertical, 2 = horizontal 
*    height       Height of button area in mm
*    width        Width of button area in mm
*
*  output 
*   (none) 
*  
**********************************************************************/


int calc_expr(int style, double height, double width)
{
/******************** Routine variable declaration  *******************/
    char
        estr[132];      /* expression string */

/******************** Beginning of Executable code *********************/


/* Create expression called Style.  Set its value to the function arg.
 * 'style'.  Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Style=%d",style);
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create expression called Body_height.  Set it equal to the input 
 * argument 'height'.  Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Body_height=%f",height);
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create expression called Body_width.  Set it equal to the input 
 * argument 'width'.  Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Body_width=%f",width);
    UF_CALL(UF_MODL_create_exp( estr ) );

/* The liquid crystal display, photocell area, and the space between
 * them have static heights.  Create the expressions and check the
 * return codes.
 */
    strcpy(estr,"lcd_height=20.0");
    UF_CALL(UF_MODL_create_exp( estr ) );

    strcpy(estr,"pc_height=8.0");
    UF_CALL(UF_MODL_create_exp( estr ) );

    strcpy(estr,"lcd_pc_height=12.0");
    UF_CALL(UF_MODL_create_exp( estr ) );

    strcpy(estr,"Border=5.0");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* If the style is vertical (1), the button arrangement is         */
/*    5 across by 9 high.                                          */
/* If it is horizontal (2), the arrangement is 8 x 6.              */
/* The space between buttons is 25% of the button width.           */
/* There is a 5 millimeter border around the calculator (4 sides). */

    /* Use the expression if/else construct to make the expression
     * Across 5 or 8.  Create the expression and check the return
     * code.
     */
    strcpy( estr, "Across=if(Style==1) (5) else (8)");
    UF_CALL(UF_MODL_create_exp( estr ) );

    /* Use the expression if/else construct to make the expression 
     * 'Up' 9 or 6.  Create the expression and check the return
     * code.
     */
    strcpy( estr, "Up=if(Style==1) (9) else (6)");
    UF_CALL(UF_MODL_create_exp( estr ) );

    /* Create 'Button_width' using the if/else construct.  It is 
     * either ((Body_width-2.0*Border)/6.0) or
     * ((Body_width-2.0*Border)/9.75).  Check the return code.
     */
    strcpy( estr, "Button_width=if(Style==1) ((Body_width-2.0*Border)/6.0)");
    strcat( estr, " else ((Body_width-2.0*Border)/9.75)");
    UF_CALL(UF_MODL_create_exp( estr ) );

    /* Create the intermediate expression Bh1 as Body_height minus
     * pc_height minus lcd_pc_height minus lcd_height minus 3 times
     * the Border.  Check the return code.
     */
    strcpy( estr, "Bh1=Body_height - pc_height - lcd_pc_height - ");
    strcat( estr, "lcd_height - 3*Border");
    UF_CALL(UF_MODL_create_exp( estr ) );

    /* Create the Button_height based on the intermediate value Bh1.
     * Use the if/else construct and use ((Bh1-2.0*Button_width)/9.0)
     * or ((Bh1-1.25*Button_width)/6.0) for the height.
     */
    strcpy( estr, "Button_height=if(Style==1) ((Bh1-2.0*Button_width)/9.0)");
    strcat( estr, "else ((Bh1-1.25*Button_width)/6.0)");
    UF_CALL(UF_MODL_create_exp( estr ) );

    /* Create the expression Button_spacing as .25 times the
     * Button_width expression.  Check the return code.
     */
    strcpy( estr, "Button_spacing=0.25*Button_width");
    UF_CALL(UF_MODL_create_exp( estr ) );

    /* Create the expression for pc_width.  It is the Body_width
     * minus twice the Border.  Check the return code.
     */
    strcpy( estr, "pc_width=Body_width - 2*Border");
    UF_CALL(UF_MODL_create_exp( estr ) );

    return (0);
}



/**********************************************************************
* calc_interpart_expr will create the interpart set of expressions that
* control construction of the calculator.
*
*  revisions 
*  rv     date      who reason for change 
*  00   01OCT01     RGK                   
*
*  input 
*   (none)
*  output 
*   (none) 
*  
**********************************************************************/


int calc_interpart_expr()
{
/******************** Routine variable declaration  *******************/
    char
        estr[132];      /* expression string */

/******************** Beginning of Executable code *********************/


/* Create interpart expression called Across.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Across=calculator::Across");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Body_height.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Body_height=calculator::Body_height");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Body_width.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Body_width=calculator::Body_width");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Border.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Border=calculator::Border");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Button_height.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Button_height=calculator::Button_height");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Button_spacing.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Button_spacing=calculator::Button_spacing");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Button_width.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Button_width=calculator::Button_width");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Style.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Style=calculator::Style");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called Up.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"Up=calculator::Up");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called lcd_height.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"lcd_height=calculator::lcd_height");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called lcd_pc_height.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"lcd_pc_height=calculator::lcd_pc_height");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called pc_height.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"pc_height=calculator::pc_height");
    UF_CALL(UF_MODL_create_exp( estr ) );

/* Create interpart expression called pc_width.  
 * Check the return flag and exit if it is nonzero.
 */
    sprintf(estr,"pc_width=calculator::pc_width");
    UF_CALL(UF_MODL_create_exp( estr ) );

    return(0);
}

