#include <string.h>
#include <math.h>
#include <uf_defs.h>
#include <uf_obj.h>
#include <uf_modl.h>
#include <uf_attr.h>
#include <uf_part.h>
#include "calcproto.h"

/**********************************************************************
* calc_model_bottom 
* 
* create the bottom half of the calculator 
*  
* revisions 
* rv   date      who   reason for change 
* 00   01NOV95   J_P 
* 01   01JUN98   J_P   Initialized some variables
* 02   14FEB03   DKF   Removed unreferenced local variable
* 03   14FEB03   DKF   Updated part attribute assignment procedure,
*                      Updated error checking
*  
* input 
*  (none) 
*  
* output 
*  calc_model_bottom        0 = success, >0 = error 
*  
**********************************************************************/


    int calc_model_bottom(void)
{
/******************** Routine variable declaration  *******************/
    char
        *edge_len_p[3],     /* Pointers to block edge length strings. */
        edge_len[3][133],   /* Strings for edge length expressions. */
        svalue[50];         /* string for attribute value */

    int ecount=0,           /* count of objects in list */
        vcount=0,           /* count of vertices in edge */
        flag=0,             /* 0 = success, >0 = error */
        i;                  /* looper */

    int allow_smooth=0,  /* allow all blends */
        allow_cliff=0,
        allow_notch=0;

    tag_t
        part,               /* part tag, for attribute assignment */
        bottom_half,        /* bottom half feature */
        body,               /* bottom_half body    */
        hollow,             /* hollow feature */
        blend1;             /* blend feature */

    uf_list_p_t
        list1=NULL,         /* list of edges */
        list2=NULL;

    double
        corner[3],          /* start corner of block */
        pt1[3],             /* coordinates of vertex */
        pt2[3],             /* coordinates of vertex */
        vrb_tol=0.001;      /* variable radius blend tolerance */


    UF_ATTR_value_t
        att_value;          /* attribute data structure */

/******************** Beginning of Executable code *********************/


/* Set up data for block creation.  The routine takes an array of
 * pointers to strings.  Set edge_len_p to the addresses of the
 * edge_len array items.
 */

    corner[0] = 0.0;
    corner[1] = 0.0;
    corner[2] = 0.0;

    edge_len_p[0] = &edge_len[0][0];
    edge_len_p[1] = &edge_len[1][0];
    edge_len_p[2] = &edge_len[2][0];

    strcpy(edge_len[0],"Body_width");
    strcpy(edge_len[1],"Body_height");
    strcpy(edge_len[2],"5.0");

/* Create the block and check the return code */
    UF_CALL( UF_MODL_create_block1(UF_NULLSIGN, corner,
             edge_len_p, &bottom_half) );

/* Obtain the body tag from the feature tag. */
    UF_CALL( UF_MODL_ask_feat_body(bottom_half, &body) );

/* Obtain the body edges. Use list1 */
    UF_CALL( UF_MODL_ask_body_edges(body, &list1) );

/* Determine the number of edges (ecount). Trap the return code */
    flag = UF_CALL( UF_MODL_ask_list_count(list1, &ecount) );

/* Delete the list and return if an error occurs */
    if( flag ) {
        UF_CALL( UF_MODL_delete_list( &list1) );
        return( flag );
    }

/* Create list2' for storing the edges to blend. 
 * Trap the return code */
    flag = UF_CALL( UF_MODL_create_list(&list2) );

/* If an error occurs, delete list1 and return. */
    if( flag ) {
        UF_CALL( UF_MODL_delete_list( &list1) );
        return( flag );
    }

/* Loop through all edges and get their vertices.  Use the edge
 * vertices to determine which edges to blend.  The edges we want have a
 * delta in Z of 5mm.  Put the id's of the edges to blend in 'list2'. 
 */
    for(i=0; i < ecount; i++) {
        tag_t
            edge;               /* edge object */

/* Get an edge from the list.  Check return code.  */
        UF_CALL( UF_MODL_ask_list_item(list1, i, &edge) );

/* Obtain the vertices.  Check return code. */
        UF_CALL( UF_MODL_ask_edge_verts(edge, pt1, pt2, &vcount) );
        if(fabs(fabs(pt1[2] - pt2[2]) - 5.0) < 0.001) {

		/* Item is placed on list2.  Check return code. */
            UF_CALL( UF_MODL_put_list_item(list2, edge) );
        }
    }

/* Blend the qualified edges with a 5 mm radius.  Check
 * return code.
 */
    UF_CALL( UF_MODL_create_blend("5.0", list2, allow_smooth,
             allow_cliff, allow_notch, vrb_tol, &blend1) );

/* Delete the previous edge lists before blending the edges of the
 * bottom face.  Since the block has been edited (blended), the first
 * edge list may no longer be valid.
 */
    UF_CALL( UF_MODL_delete_list(&list1) );
    UF_CALL( UF_MODL_delete_list(&list2) );

/* Get the new body edges. Use list1 */
    UF_CALL( UF_MODL_ask_body_edges(body, &list1) );

/* Obtain the number of objects on the edge list (ecount). 
 * Trap the retrun code */
    flag = UF_CALL( UF_MODL_ask_list_count(list1, &ecount) );

/* Delete the list and return if an error occurs */
    if( flag ) {
        UF_CALL( UF_MODL_delete_list( &list1) );
        return( flag );
    }

/* Create 'list2' for storing edges to blend. Trap return code.
 */
    flag = UF_CALL( UF_MODL_create_list(&list2) );

/* If an error occurs, delete list1 and return. */
    if( flag ) {
        UF_CALL( UF_MODL_delete_list( &list1) );
        return( flag );
    }


    for(i=0; i < ecount; i++) { 
        tag_t
            edge;               /* edge object */

		/* Obtain an edge from the list.  Check return code. */
        UF_CALL( UF_MODL_ask_list_item(list1, i, &edge) );

/* Get the edge vertices and check them.  We want bottom edges
 * with Z of 0.
 */
        UF_CALL( UF_MODL_ask_edge_verts(edge, pt1, pt2, &vcount) );
        if((fabs(pt1[2]) < 0.001)
            && (fabs(pt2[2]) < 0.001)) { 
			/* Item is placed on list2.  Check return code. */
            UF_CALL( UF_MODL_put_list_item(list2, edge) );
        }
    }

/* Blend the qualified edges with a 2 mm radius.  Check
 * return code.
 */
    UF_CALL( UF_MODL_create_blend("2.0", list2, allow_smooth,
             allow_cliff, allow_notch, vrb_tol, &blend1) );
  
/* Delete the lists used for the blend. */
    UF_CALL( UF_MODL_delete_list(&list1) );
    UF_CALL( UF_MODL_delete_list(&list2) );

/* Get the body faces. Use list1 */
    UF_CALL( UF_MODL_ask_body_faces(body, &list1) );

/* Create list2 for storing the face to hollow. */
    UF_CALL( UF_MODL_create_list(&list2) );

/* Get the count of faces (ecount).  Loop through the faces and
 * check the Z coordinate of the center and the normal.
 */
    UF_CALL( UF_MODL_ask_list_count(list1, &ecount) );
    for(i=0; i < ecount; i++) {
        tag_t
            face;               /* face object      */

        int ftype,              /* face type */
            dsense;             /* face direction sense */

        double
            nrm[3],             /* Face normal direction */
            box[6],             /* face boundary */
            rad1,               /* face major radius */
            rad2;               /* face minor radius */

		/* Obtain an face from the list.  Check return code. */
        UF_CALL( UF_MODL_ask_list_item(list1, i, &face) );

		/* Obtain data about the face.  Check return code. */
        UF_CALL( UF_MODL_ask_face_data(face, &ftype, pt1, nrm, box,
                 &rad1, &rad2, &dsense) );
        if((fabs(pt1[2] - 5.0) < 0.001)
            && (fabs(nrm[2] - 1.0) < 0.001)) {
			/* Put face on list2.  Check return code.  Break from the loop. */
            UF_CALL( UF_MODL_put_list_item(list2, face) );
            break;
        }
    }

/* Perform hollow operation.  Use a 1mm thickness.
 * Check return code.
 */
    UF_CALL( UF_MODL_create_hollow("1.0", list2, &hollow) );

/* Delete the lists used to create the hollow. */
    UF_CALL( UF_MODL_delete_list(&list1) );
    UF_CALL( UF_MODL_delete_list(&list2) );

/* Assign the name BOTTOM_HALF to the body.  */
    UF_CALL( UF_OBJ_set_name( body, "BOTTOM_HALF" ) );

/*YYY  Attribute code will be added after the model code is completed */
/* Add part number and description attributes to the part.  Obtain the 
 * display part tag. .
 * For UF_ATTR_assign, use the titles
 * "PARTNUMBER" and "DESCRIPTN"
 * and the values shown below.
 */
    part = UF_PART_ask_display_part();
    att_value.type = UF_ATTR_string;
    strcpy(svalue,"002");    /* First call for the part number */
    att_value.value.string = svalue;
    UF_CALL( UF_ATTR_assign( part, "PARTNUMBER", att_value) );
    strcpy(svalue,"BOTTOM");    /* Second call for the description. */
    UF_CALL( UF_ATTR_assign( part, "DESCRIPTN", att_value) );

    return (0);
}
