#include <string.h>
#include <math.h>
#include <uf_defs.h>
#include <uf_obj.h>
#include <uf_modl.h>
#include <uf_attr.h>
#include <uf_part.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_model_button 
*
* create the button of the calculator 
*
* revisions 
* rv   date      who   reason for change 
* 00   01NOV95   J_P 
* 01   01AUG97   J_P   changed some variable names and limited scope of
*                      other variables.
* 02   01JUN98   J_P   initialized some varibles
* 03   14FEB03   DKF   Updated part attribute assignment procedure
*
* input 
*  (none) 
*
* output 
*  calc_model_button       0 = success, >0 = error 
*
*  
**********************************************************************/

    int calc_model_button(void)
{
/******************** Routine variable declaration  *******************/
    char
        edge_len[3][133],   /* edge length expression strings. */
        *edge_len_p[3],     /* pointers to edge length exps. */
        svalue[30];         /* string for attribute value */

    int count=0,            /* count of objects in list */
        i;                  /* looper */

    tag_t
        part,               /* part tag, for attribute assignment */
        button,             /* button feature */
        face1,              /* first through face for slot */
        face2,              /* second through face for slot */
        face3,              /* placement face for slot */
        slot,               /* slot feature */
        body;               /* button solid body */

    uf_list_p_t
       list1=NULL;          /* list of edges */

    double
        corner[3],          /* start corner of block */
        location[3],        /* location for hole or slot feature */
        axis[3],            /* tool axis             */
        x_dir[3],           /* feature direction     */
        button_height,      /* Result from evaluating exp. */
        button_width;       /* Result from evaluating exp. */

    UF_ATTR_value_t
        att_value;          /* attribute data structure */

/******************** Beginning of Executable code *********************/


/* Set the location for the button block and set the pointers to the 
 * strings with the expressions for size. 
 */
    corner[0] = 5.0;
    corner[1] = 5.0;
    corner[2] = 7.0;

    edge_len_p[0] = &edge_len[0][0];
    edge_len_p[1] = &edge_len[1][0];
    edge_len_p[2] = &edge_len[2][0];

    strcpy(edge_len[0],"Button_width");
    strcpy(edge_len[1],"Button_height");
    strcpy(edge_len[2],"5.0");

/* Create the block and check the return code.  */
    UF_CALL( UF_MODL_create_block1(UF_NULLSIGN, corner,
             edge_len_p, &button) );

/* Get the faces of the feature just created.  Determine the
 * number of faces (count).  Check return codes.
 */
    UF_CALL( UF_MODL_ask_feat_faces(button, &list1) );
    UF_CALL( UF_MODL_ask_list_count(list1, &count) );

/* Loop through the faces.  Use the face data normal information to
 * determine the faces for the slot creation.  face3 is the face for the
 * slot (it will have a +Z normal).  face1 and face2 are the 'thru'
 * faces;  they will have +/- X direction normals.
 */

    for(i=0; i < count; i++) {
        tag_t
            face;               /* face object      */

        int ftype,              /* face type */
            dsense;             /* face normal sense */

        double
            pt1[3],             /* coordinates of vertex */
            nrm[3],             /* coordinates of vertex */
            box[6],             /* face boundary         */
            rad1,               /* face major radius     */
            rad2;               /* face minor radius     */

/* Obtain a face from the list.  Check the return code. */
        UF_CALL( UF_MODL_ask_list_item(list1, i, &face) );

/* Get the face data for the normal information.
 * Check return code.
 */
        UF_CALL( UF_MODL_ask_face_data(face, &ftype, pt1, nrm, box,
               &rad1, &rad2, &dsense) );
        if(fabs(nrm[0] + 1.0) < 0.001) {
            face1 = face;
        }
        if(fabs(nrm[0] - 1.0) < 0.001) {
            face2 = face;
        }
        if(fabs(nrm[2] - 1.0) < 0.001) {
            face3 = face;
        }
    }

/* Delete the list of faces.  */
    UF_CALL( UF_MODL_delete_list(&list1) );

/* Evaluate the expressions for Button_width and Button_height.
 * (store the results in variables button_width and button_height)
 * and use them to calculate the slot location.
 */
    UF_CALL( UF_MODL_eval_exp("Button_width", &button_width) );
    UF_CALL( UF_MODL_eval_exp("Button_height", &button_height) );
    location[0] = 0.5*button_width + 5.0;
    location[1] = 0.6*button_height + 5.0;
    location[2] = 12.0;

/* Set the axis for the slot normal and the direction for the slot. */
    axis[0] = 0.0;
    axis[1] = 0.0;
    axis[2] = 1.0;

    x_dir[0] = 1.0;
    x_dir[1] = 0.0;
    x_dir[2] = 0.0;

/* Create a slot with a width of .25 and a depth of .5 mm.  The
 * slot length will be determined by the faces on the button edge.  Use
 * the "Button_width" expression for the length calling argument.
 */
    UF_CALL( UF_MODL_create_rect_slot(location, axis, x_dir,
           ".25", ".5", "Button_width", face3, face1, face2, &slot) );

/* Get the button feature's body tag and
 * name the body "BUTTON".
 */
    UF_CALL( UF_MODL_ask_feat_body(button, &body) );
    UF_CALL( UF_OBJ_set_name( body, "BUTTON") );

/*YYY  Attribute code will be added after the model code is completed */
/* Add part number and description attributes to the part.  Obtain the 
 * display part tag. .
 * For UF_ATTR_assign, use the titles
 * "PARTNUMBER" and "DESCRIPTN"
 * and the values shown below.
 */
    part = UF_PART_ask_display_part();
    att_value.type = UF_ATTR_string;
    strcpy(svalue,"003");     /* First call is the part number. */
    att_value.value.string = svalue;
    UF_CALL( UF_ATTR_assign( part, "PARTNUMBER", att_value ) );
    strcpy(svalue,"BUTTON");  /* Second call is the description. */
    UF_CALL( UF_ATTR_assign( part, "DESCRIPTN", att_value ) );

    return (0);
}
