/************************************************************************
*   calc_part
*
* This routine checks for open parts.  If parts are open, it allows the
* user to save and close or close the opened parts.  The program will try
* to open the calculator part  (calculator.prt).
*
*  input
*    (none)
*
*  output
*    calc_part     0 = success, else failure
*  revisions
*    rv    date        who    reason for change
*    00    01NOV95     J_P   
*    01    14FEB03     DKF    Update error checking, add Windows path
*
************************************************************************/


#include <string.h>
#include <uf_defs.h>
#include <uf.h>
#include <uf_part.h>
#include <uf_ui.h>
#include "calcproto.h"

#define  WAIT   ((int) 1)    /* For uc1601 to wait for OK after msg.    */
#define  NOWAIT ((int) 0)    /* For uc1601 to display in status line.   */

    int calc_part()
{
/********************* Routine variable declaration  ********************/

    char
        estr[132];			           /* general purpose string */

    int numparts,            /* number of currently loaded parts */
        resp;                /* user response */

    tag_t
        parent;              /* tag of parent part */

    UF_PART_load_status_t
        estat;               /* structure for load part */

/******************** Beginning of Executable code **********************/
 

	/*XXX DONE XXX Determine if any parts are open (check num_parts) */
    numparts = UF_PART_ask_num_parts();

    if(numparts > 0)
    {  
        char
            option[2][38];      /* menu options */

        int deflt,               /* menu default */
            errcount,            /* count of errors from
                                    UF_PART_save_all */
            *errcodes;           /* error codes from UF_PART_save_all */

        tag_t
            *errtags;            /* tags of parts that failed to save */

        /*XXX DONE XXX Put up 2 options;  "Save and close" or "close without
         * saving".  Use the 'Close' option as the default.
         */
        strcpy( option[0], "Save and Close all parts" );
        strcpy( option[1], "Close all parts w/o save" );
        deflt = 2;
        resp = uc1603( "All parts must be closed", deflt, option, 2);

		/* Dismiss the DA2 in preparation for the styler call. */
        UF_CALL( UF_UI_dismiss_dialog_area_2() );

		/* Return a 1 if Back or Cancel was picked, or perform selected
		 * option.
		 */
        if( resp < 3 )
            return( 1 );

		/* Check the resp and perform the appropriate action based on the
		 * value.  The opened parts will always be closed by going through
		 * this section of code. 
		 * XXX DONE XXX Call the part save all function if
		 * the save option was selected.  Check for errors on the save and
		 * call the free routines if errors occurred.  */
        if( resp == 5 ) {
            resp = UF_CALL( UF_PART_save_all(&errcount, &errtags,
                            &errcodes) );
            if( resp )
                return( 1 );
            if( errcount ) {
                UF_free( errtags );
                UF_free( errcodes );
                return( 2 );
            }
         }

		/* XXX DONE XXX Call the part
		 * close all function regardless for both cases.  
		 */
         UF_CALL( UF_PART_close_all( ) );
    }

/*XXX Retrieve calculator.prt using resp = UF_PART_open.  Display an
 * error dialog and return an non-zero if an error occurs.
 */
	/* Windows */
    strcpy( estr, ".\\calculator.prt");
	/* Unix 
    strcpy( estr, "./calculator.prt");*/
    UF_CALL( UF_PART_open( estr, &parent, &estat ) );

    if( estat.n_parts ) {
        uc1601(" Calculator part had components that did not load",1);
        uc1601(" The part should be an empty part!!",1);
        UF_free( estat.statuses);
        UF_free_string_array( estat.n_parts, estat.file_names );
        return( 1 );
    }

    return (0);
}
